﻿import * as React from 'react'
import {
    withValidation,
    IWithValidationInjectedProps,
    ValidationFeedback,
    Validator,
    Validators,
    IThemeColors,
    ITIReactContext
} from '../..'
import Select from 'react-select'
import { partition, flatten } from 'lodash'
import * as Color from 'color'

// Style the select to match Bootstrap form-control inputs
export function getSelectStyles(
    valid: boolean,
    showValidation: boolean,
    themeColors: IThemeColors,
    width?: number
) {
    const disabledDarkenBy = 0.15

    return {
        control: (base: any, state: any) => {
            const styles: any = { ...base }

            if (typeof width === 'number') styles.width = width

            styles.borderColor = '#ced4da' // $gray-400

            if (state.isDisabled) {
                return {
                    ...styles,
                    backgroundColor: '#e9ecef' // $gray-200
                }
            }

            const primaryColor = new Color(themeColors.primary)
            const dangerColor = new Color(themeColors.danger)
            const successColor = new Color(themeColors.success)

            styles.backgroundColor = 'white'

            if (showValidation) {
                const borderColor = valid ? successColor : dangerColor

                styles.borderColor = borderColor.toString()
                styles['&:hover'].borderColor = borderColor.toString()
            }

            if (state.isFocused) {
                const borderColor = primaryColor.lighten(0.25)
                const boxShadowColor = primaryColor.alpha(0.25)

                styles.borderColor = borderColor.toString()
                styles['&:hover'].borderColor = borderColor.toString()
                styles.boxShadow = `0 0 0 0.2rem ${boxShadowColor.toString()}`
            }

            return styles
        },
        placeholder: (base: any, state: any) => {
            if (state.isDisabled) return base

            return {
                ...base,
                color: themeColors.inputPlaceholder
            }
        },
        dropdownIndicator: (base: any, state: any) => {
            if (state.isDisabled) {
                return {
                    ...base,
                    color: new Color(base.color).darken(disabledDarkenBy).toString()
                }
            }

            return base
        },
        indicatorSeparator: (base: any, state: any) => {
            if (state.isDisabled) {
                return {
                    ...base,
                    backgroundColor: new Color(base.backgroundColor)
                        .darken(disabledDarkenBy)
                        .toString()
                }
            }

            return base
        }
    }
}

export function getNonGroupOptions(options: (IOption | IGroupOption)[]): IOption[] {
    let [groupOptions, nonGroupOptions] = partition(
        options,
        o => typeof (o as any).value === 'undefined'
    ) as [IGroupOption[], IOption[]]

    return [...nonGroupOptions, ...flatten<IOption>(groupOptions.map(go => go.options))]
}

export type SelectValue = string | number | null

export interface IOption {
    value: string | number
    label: string
}

export interface IGroupOption {
    label: string
    options: IOption[]
}

interface IValidatedSelectOwnProps extends React.Props<any> {
    options: (IOption | IGroupOption)[]
    isClearable?: boolean
    enabled?: boolean
    placeholder?: string
    className?: string
    width?: number
}

type IValidatedSelectProps = IValidatedSelectOwnProps &
    IWithValidationInjectedProps<SelectValue>

class _ValidatedSelect extends React.Component<IValidatedSelectProps> {
    static defaultProps: Pick<IValidatedSelectProps, 'enabled' | 'isClearable'> = {
        enabled: true,
        isClearable: false
    }

    onChange = (option: IOption | null, { action }: { action: string }) => {
        // option will be an array if the user presses backspace

        const { onChange, isClearable } = this.props

        // This is so that if isClearable = false, you don't have to worry about ValidatedSelect's
        // onChange returning null. pop-value doesn't happen often so this will hopefully avoid
        // some bugs.
        if (!isClearable && action === 'pop-value') return

        // Be careful with the conditional - option.value could be 0
        let newValue: SelectValue = null
        if (option && option.value !== null) {
            newValue = option.value
        }

        onChange(newValue)
    }

    render() {
        const {
            options,
            value,
            valid,
            invalidFeedback,
            showValidation,
            name,
            isClearable,
            enabled,
            placeholder,
            className,
            width
        } = this.props

        const nonGroupOptions = getNonGroupOptions(options)

        let selectValue: IOption | null = null

        // Be careful in conditional - value can be 0
        if (value !== null) {
            selectValue = nonGroupOptions.find(o => o.value === value)!
        }

        return (
            <ValidationFeedback
                valid={valid}
                invalidFeedback={invalidFeedback}
                showValidation={showValidation}
            >
                <ITIReactContext.Consumer>
                    {data => (
                        <Select
                            name={name}
                            className={className}
                            options={options}
                            placeholder={placeholder}
                            value={selectValue}
                            onChange={this.onChange}
                            isClearable={isClearable}
                            isDisabled={!enabled}
                            styles={getSelectStyles(
                                valid,
                                showValidation,
                                data.themeColors,
                                width
                            )}
                        />
                    )}
                </ITIReactContext.Consumer>
            </ValidationFeedback>
        )
    }
}

const options = {
    defaultValue: null
}

export const ValidatedSelect = withValidation<IValidatedSelectOwnProps, SelectValue>(
    options
)(_ValidatedSelect)

function required(): Validator<SelectValue> {
    return (value: SelectValue) => ({
        valid: value !== null,
        invalidFeedback: Validators.required()('').invalidFeedback
    })
}

export const SelectValidators = {
    required
}
